﻿using CoreLibrary.Exceptions;
using Xunit;

namespace UnitTesting
{
    public class CodeInvalideExceptionUT
    {
        [Fact]
        public void ExceptionDefaut()
        {
            Assert.ThrowsAsync<CodeInvalideException>(() => throw new CodeInvalideException());
        }

        [Fact]
        public void ExceptionAttributs()
        {
            Assert.ThrowsAsync<CodeInvalideException>(() => throw new CodeInvalideException(3, 4));

            try
            {
                throw new CodeInvalideException(3, 4);
            }
            catch (CodeInvalideException e)
            {
                Assert.Contains("3", e.Message);
                Assert.Contains("4", e.Message);
            }
        }

        [Fact]
        public void ExceptionMessage()
        {
            string message = "Mon super gros problème.";

            Assert.ThrowsAsync<CodeInvalideException>(() => throw new CodeInvalideException(message));

            try
            {
                throw new CodeInvalideException(message);
            }
            catch(CodeInvalideException e)
            {
                Assert.Equal(message, e.Message);
            }
        }

        [Fact]
        public void ExceptionMessageEtException()
        {
            string message = "Mon super gros problème.";
            string message2 = "Pas de chance...";
            InvalidOperationException parent = new InvalidOperationException(message2);

            Assert.ThrowsAsync<CodeInvalideException>(() => throw new CodeInvalideException(message, parent));

            try
            {
                throw new CodeInvalideException(message, parent);
            }
            catch (CodeInvalideException e)
            {
                Assert.Equal(message, e.Message);
                Assert.NotNull(e.InnerException);
                Assert.IsType<InvalidOperationException>(e.InnerException);
                Assert.Equal(message2, e.InnerException.Message);
            }
        }

    }
}
