﻿using CoreLibrary.Exceptions;
using Xunit;

namespace UnitTesting
{
    public class TailleCodeExceptionUT
    {
        [Fact]
        public void ExceptionDefaut()
        {
            Assert.ThrowsAsync<TailleCodeException>(() => throw new TailleCodeException());
        }

        [Fact]
        public void ExceptionAttributs()
        {
            Assert.ThrowsAsync<TailleCodeException>(() => throw new TailleCodeException(-3));

            try
            {
                throw new TailleCodeException(-3);
            }
            catch (TailleCodeException e)
            {
                Assert.Contains("-3", e.Message);
            }
        }

        [Fact]
        public void ExceptionMessage()
        {
            string message = "Mon super gros problème.";

            Assert.ThrowsAsync<TailleCodeException>(() => throw new TailleCodeException(message));

            try
            {
                throw new TailleCodeException(message);
            }
            catch(TailleCodeException e)
            {
                Assert.Equal(message, e.Message);
            }
        }

        [Fact]
        public void ExceptionMessageEtException()
        {
            string message = "Mon super gros problème.";
            string message2 = "Pas de chance...";
            InvalidOperationException parent = new InvalidOperationException(message2);

            Assert.ThrowsAsync<TailleCodeException>(() => throw new TailleCodeException(message, parent));

            try
            {
                throw new TailleCodeException(message, parent);
            }
            catch (TailleCodeException e)
            {
                Assert.Equal(message, e.Message);
                Assert.NotNull(e.InnerException);
                Assert.IsType<InvalidOperationException>(e.InnerException);
                Assert.Equal(message2, e.InnerException.Message);
            }
        }

    }
}
